/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/DungeonKeyModel.h"
#include "convergence/DungeonKeyView.h"
#include "convergence/GraphicsConstants.h"
#include <math.h>
#include <stddef.h>
#include <stdlib.h>

struct vertex_p2f_t2f {
	GLfloat position[2];
	GLfloat texCoords[2];
};

DungeonKeyView * DungeonKeyView_create(compat_type(DungeonKeyModel *) modelPtr, ResourceManager * resourceManager) {
	stemobject_create_implementation(DungeonKeyView, init, modelPtr, resourceManager)
}

void DungeonKeyView_init(compat_type(DungeonKeyView *) selfPtr, compat_type(DungeonKeyModel *) modelPtr, ResourceManager * resourceManager) {
	DungeonKeyView * self = selfPtr;
	struct vertex_p2f_t2f vertices[4] = {
		{{-0.5f * KEY_RADIUS, -1.0f * KEY_RADIUS}, { 1.0f / 64.0f,  2.0f / 64.0f}},
		{{ 0.5f * KEY_RADIUS, -1.0f * KEY_RADIUS}, {32.0f / 64.0f,  2.0f / 64.0f}},
		{{ 0.5f * KEY_RADIUS,  1.0f * KEY_RADIUS}, {32.0f / 64.0f, 63.0f / 64.0f}},
		{{-0.5f * KEY_RADIUS,  1.0f * KEY_RADIUS}, { 1.0f / 64.0f, 63.0f / 64.0f}}
	};
	GLushort indexes[6] = {
		0, 1, 2,
		2, 3, 0
	};
	
	GameObjectView_init(self, modelPtr);
	self->dispose = DungeonKeyView_dispose;
	self->draw = DungeonKeyView_draw;
	
	self->sortOrder = -1;
	
	self->resourceManager = resourceManager;
	
	glGenBuffers(1, &self->vboID);
	glGenBuffers(1, &self->eboID);
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(struct vertex_p2f_t2f) * 4, vertices, GL_STATIC_DRAW);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLushort) * 6, indexes, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
	self->texture = self->resourceManager->referenceResource(self->resourceManager, "texture", "textures/gem.json");
}

void DungeonKeyView_dispose(compat_type(DungeonKeyView *) selfPtr) {
	DungeonKeyView * self = selfPtr;
	
	glDeleteBuffers(1, &self->vboID);
	glDeleteBuffers(1, &self->eboID);
	self->resourceManager->releaseResource(self->resourceManager, "texture", "textures/gem.json");
	GameObjectView_dispose(self);
}

void DungeonKeyView_draw(compat_type(DungeonKeyView *) selfPtr, Matrix modelviewMatrix, double deltaTime) {
	DungeonKeyView * self = selfPtr;
	DungeonKeyModel * model = (DungeonKeyModel *) self->model;
	uint32_t color;
	
	Matrix_translate(&modelviewMatrix, model->position.x, model->position.y, 0.0f);
	
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glVertexPointer(2, GL_FLOAT, sizeof(struct vertex_p2f_t2f), (void *) offsetof(struct vertex_p2f_t2f, position));
	glTexCoordPointer(2, GL_FLOAT, sizeof(struct vertex_p2f_t2f), (void *) offsetof(struct vertex_p2f_t2f, texCoords));
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelviewMatrix.m);
	if (model->keyID < 0 || model->keyID >= KEY_COLOR_COUNT) {
		color = 0xFFFFFF;
	} else {
		color = KEY_COLORS[model->keyID];
	}
	glColor4ub(color >> 16 & 0xFF, color >> 8 & 0xFF, color & 0xFF, 0xFF);
	self->texture->activate(self->texture);
	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, 0);
	self->texture->deactivate(self->texture);
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}
