/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/DungeonRoomStateModel.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

DungeonRoomStateModel * DungeonRoomStateModel_create(Atom name, unsigned int tileCountX, unsigned int tileCountY, struct DungeonTile * tiles, const char * tileCanvasFileName) {
	stemobject_create_implementation(DungeonRoomStateModel, init, name, tileCountX, tileCountY, tiles, tileCanvasFileName)
}

// MASSIVE STINKING DISGUSTING HACK, but it works!
static void generateEdgeTiles(const struct DungeonTile * inTiles, struct DungeonTile * outTiles, unsigned int tileCountX, unsigned int tileCountY) {
	unsigned int x, y;
	bool edgeTopLeft, edgeTop, edgeTopRight, edgeLeft, edgeRight, edgeBottomLeft, edgeBottom, edgeBottomRight;
	const char * prefixes[2] = {
		"wall",
		"floor"
	};
	char tileName[16];
	
	for (y = 0; y < tileCountY; y++) {
		for (x = 0; x < tileCountX; x++) {
			edgeTopLeft = edgeTop = edgeTopRight = edgeLeft = edgeRight = edgeBottomLeft = edgeBottom = edgeBottomRight = false;
			if (y > 0) {
				if (x > 0) {
					edgeBottomLeft = inTiles[(y - 1) * tileCountX + x - 1].type != inTiles[y * tileCountX + x].type;
				}
				edgeBottom = inTiles[(y - 1) * tileCountX + x].type != inTiles[y * tileCountX + x].type;
				if (x < tileCountX - 1) {
					edgeBottomRight = inTiles[(y - 1) * tileCountX + x + 1].type != inTiles[y * tileCountX + x].type;
				}
			}
			if (x > 0) {
				edgeLeft = inTiles[y * tileCountX + x - 1].type != inTiles[y * tileCountX + x].type;
			}
			if (x < tileCountX - 1) {
				edgeRight = inTiles[y * tileCountX + x + 1].type != inTiles[y * tileCountX + x].type;
			}
			if (y < tileCountY - 1) {
				if (x > 0) {
					edgeTopLeft = inTiles[(y + 1) * tileCountX + x - 1].type != inTiles[y * tileCountX + x].type;
				}
				edgeTop = inTiles[(y + 1) * tileCountX + x].type != inTiles[y * tileCountX + x].type;
				if (x < tileCountX - 1) {
					edgeTopRight = inTiles[(y + 1) * tileCountX + x + 1].type != inTiles[y * tileCountX + x].type;
				}
			}
			
			if (edgeBottomLeft && (edgeLeft || edgeBottom)) {edgeBottomLeft = false;}
			if (edgeBottomRight && (edgeRight || edgeBottom)) {edgeBottomRight = false;}
			if (edgeTopLeft && (edgeLeft || edgeTop)) {edgeTopLeft = false;}
			if (edgeTopRight && (edgeRight || edgeTop)) {edgeTopRight = false;}
			
			sprintf(tileName, "%s_%d%d%d%d%d%d%d%d", prefixes[inTiles[y * tileCountX + x].type], edgeLeft, edgeBottom, edgeTop, edgeRight, edgeBottomLeft, edgeBottomRight, edgeTopLeft, edgeTopRight);
			outTiles[y * tileCountX + x].canvasTileID = Atom_fromString(tileName);
		}
	}
}

static void sharedInit(DungeonRoomStateModel * self) {
	StemObject_init(self);
	self->dispose = DungeonRoomStateModel_dispose;
	self->copy = DungeonRoomStateModel_copy;
	self->addObject = DungeonRoomStateModel_addObject;
	self->removeObject = DungeonRoomStateModel_removeObject;
	self->removeObjectAtIndex = DungeonRoomStateModel_removeObjectAtIndex;
	self->softReset = DungeonRoomStateModel_softReset;
	
	self->eventDispatcher = EventDispatcher_create(self);
}

void DungeonRoomStateModel_init(compat_type(DungeonRoomStateModel *) selfPtr, Atom name, unsigned int tileCountX, unsigned int tileCountY, struct DungeonTile * tiles, const char * tileCanvasFileName) {
	DungeonRoomStateModel * self = selfPtr;
	
	sharedInit(self);
	self->name = name;
	self->objectCount = 0;
	self->objectListSize = 1;
	self->objects = malloc(sizeof(GameObjectModel *) * self->objectListSize);
	self->tileCountX = tileCountX;
	self->tileCountY = tileCountY;
	self->tiles = malloc(sizeof(struct DungeonTile) * tileCountX * tileCountY);
	memcpy(self->tiles, tiles, sizeof(struct DungeonTile) * tileCountX * tileCountY);
	self->tileCanvasFileName = malloc(strlen(tileCanvasFileName) + 1);
	strcpy(self->tileCanvasFileName, tileCanvasFileName);
	
	generateEdgeTiles(tiles, self->tiles, tileCountX, tileCountY);
}

void DungeonRoomStateModel_initCopy(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(DungeonRoomStateModel *) originalPtr) {
	DungeonRoomStateModel * self = selfPtr, * original = originalPtr;
	size_t objectIndex;
	
	sharedInit(self);
	self->name = original->name;
	self->objectCount = original->objectCount;
	self->objectListSize = original->objectListSize;
	self->objects = malloc(sizeof(GameObjectModel *) * self->objectListSize);
	for (objectIndex = 0; objectIndex < self->objectCount; objectIndex++) {
		self->objects[objectIndex] = original->objects[objectIndex]->copy(original->objects[objectIndex]);
		self->objects[objectIndex]->stateModel = self;
	}
	self->tileCountX = original->tileCountX;
	self->tileCountY = original->tileCountY;
	self->tiles = malloc(sizeof(struct DungeonTile) * self->tileCountX * self->tileCountY);
	memcpy(self->tiles, original->tiles, sizeof(struct DungeonTile) * self->tileCountX * self->tileCountY);
	self->tileCanvasFileName = malloc(strlen(original->tileCanvasFileName) + 1);
	strcpy(self->tileCanvasFileName, original->tileCanvasFileName);
}

void DungeonRoomStateModel_dispose(compat_type(DungeonRoomStateModel *) selfPtr) {
	DungeonRoomStateModel * self = selfPtr;
	size_t objectIndex;
	
	for (objectIndex = 0; objectIndex < self->objectCount; objectIndex++) {
		self->objects[objectIndex]->dispose(self->objects[objectIndex]);
	}
	free(self->objects);
	free(self->tiles);
	free(self->tileCanvasFileName);
	self->eventDispatcher->dispose(self->eventDispatcher);
	StemObject_dispose(self);
}

compat_type(DungeonRoomStateModel *) DungeonRoomStateModel_copy(compat_type(DungeonRoomStateModel *) selfPtr) {
	stemobject_copy_implementation(DungeonRoomStateModel, initCopy)
}

void DungeonRoomStateModel_addObject(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(GameObjectModel *) object) {
	DungeonRoomStateModel * self = selfPtr;
	
	if (self->objectListSize <= self->objectCount) {
		self->objectListSize = self->objectListSize == 0 ? 1 : self->objectListSize * 2;
		self->objects = realloc(self->objects, sizeof(GameObjectModel *) * self->objectListSize);
	}
	self->objects[self->objectCount++] = object;
	self->eventDispatcher->dispatchEvent(self->eventDispatcher, Atom_fromString(DUNGEON_ROOM_STATE_EVENT_OBJECT_ADDED), object);
}

void DungeonRoomStateModel_removeObject(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(GameObjectModel *) object, bool dispose) {
	DungeonRoomStateModel * self = selfPtr;
	size_t objectIndex;
	
	for (objectIndex = 0; objectIndex < self->objectCount; objectIndex++) {
		if (self->objects[objectIndex] == object) {
			self->removeObjectAtIndex(self, objectIndex, dispose);
			break;
		}
	}
}

void DungeonRoomStateModel_removeObjectAtIndex(compat_type(DungeonRoomStateModel *) selfPtr, size_t objectIndex, bool dispose) {
	DungeonRoomStateModel * self = selfPtr;
	
	if (objectIndex < self->objectCount) {
		GameObjectModel * object;
		
		object = self->objects[objectIndex];
		self->objectCount--;
		for (; objectIndex < self->objectCount; objectIndex++) {
			self->objects[objectIndex] = self->objects[objectIndex + 1];
		}
		self->eventDispatcher->dispatchEvent(self->eventDispatcher, Atom_fromString(DUNGEON_ROOM_STATE_EVENT_OBJECT_REMOVED), object);
		if (dispose) {
			object->dispose(object);
		}
	}
}

void DungeonRoomStateModel_softReset(compat_type(DungeonRoomStateModel *) selfPtr) {
	DungeonRoomStateModel * self = selfPtr;
	size_t objectIndex;
	
	objectIndex = self->objectCount;
	do {
		objectIndex--;
		if (self->objects[objectIndex]->type == GAME_OBJECT_TYPE_PROJECTILE || self->objects[objectIndex]->type == GAME_OBJECT_TYPE_SWORD_SWING) {
			self->removeObjectAtIndex(self, objectIndex, true);
		} else {
			self->objects[objectIndex]->softReset(self->objects[objectIndex]);
		}
	} while (objectIndex > 0);
}
