/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __ENEMY_MODEL_H__
#define __ENEMY_MODEL_H__

typedef struct EnemyModel EnemyModel;

#include "convergence/AIMath.h"
#include "convergence/InventoryModel.h"
#include "convergence/MovingEntityModel.h"
#include <stdlib.h>

#define ENEMY_EVENT_ATTACK "ENEMY_EVENT_ATTACK"
#define ENEMY_EVENT_SHOOT "ENEMY_EVENT_SHOOT"
#define ENEMY_EVENT_HIT "ENEMY_EVENT_HIT"
#define ENEMY_EVENT_DEATH "ENEMY_EVENT_DEATH"

enum EnemyType {
	ENEMY_TYPE_BERSERKER,
	ENEMY_TYPE_RANGER,
	ENEMY_TYPE_FIRST_BOSS
};

enum EnemyAIState {
	ENEMY_AI_STATE_PATROL,
	ENEMY_AI_STATE_HIT_REACT,
	ENEMY_AI_STATE_PURSUE,
	ENEMY_AI_STATE_ATTACK,
	ENEMY_AI_STATE_LOST
};

#define ENEMY_AI_UPDATE_INTERVAL 15

#define EnemyModel_structContents \
	MovingEntityModel_structContents \
	\
	Vector2 initialPosition; \
	Direction initialFacing; \
	enum EnemyType enemyType; \
	int rank; \
	InventoryModel * inventory; \
	size_t patrolWaypointCount; \
	Vector2 * patrolWaypoints; \
	size_t nextPatrolWaypointIndex; \
	int maxHealth; \
	int health; \
	float knockbackResistance; \
	int immunity; \
	bool lastHitBlocked; \
	int stun; \
	int aiUpdateCooldown; \
	bool canSeePlayer; \
	Vector2 lastKnownPlayerPosition; \
	struct travelPath travelPath; \
	size_t nextTravelPathNodeIndex; \
	float passiveVisionDistance; \
	float activeVisionDistance; \
	float stealthVisionDistance; \
	enum EnemyAIState aiState; \
	int hitReactCooldown; \
	int lostGiveUpCooldown; \
	\
	void (* protected_ivar(aiUpdate))(compat_type(EnemyModel *) selfPtr); \
	void (* setAIState)(compat_type(EnemyModel *) self, enum EnemyAIState state); \
	void (* attack)(compat_type(EnemyModel *) selfPtr); \
	void (* protected_ivar(takeDamage))(compat_type(EnemyModel *) selfPtr, int amount, Direction fromDirection); \
	void (* protected_ivar(moveTowardPoint))(compat_type(EnemyModel *) selfPtr, Vector2 point, float slop);

struct EnemyModel {
	EnemyModel_structContents
};

void EnemyModel_init(compat_type(EnemyModel *) selfPtr, DungeonRoomStateModel * stateModel, int rank, int health, Vector2 position, Direction facing, size_t inventoryItemCount, struct InventoryModel_item * inventoryItems, size_t patrolWaypointCount, Vector2 * patrolWaypoints);
void EnemyModel_initCopy(compat_type(EnemyModel *) selfPtr, compat_type(EnemyModel *) originalPtr);
void EnemyModel_dispose(compat_type(EnemyModel *) selfPtr);

compat_type(EnemyModel *) EnemyModel_copy(compat_type(EnemyModel *) selfPtr);
void EnemyModel_update(compat_type(EnemyModel *) selfPtr);
void EnemyModel_softReset(compat_type(EnemyModel *) selfPtr);
void EnemyModel_aiUpdate(compat_type(EnemyModel *) selfPtr);
bool EnemyModel_atRest(compat_type(EnemyModel *) selfPtr);
bool EnemyModel_wouldCollide(compat_type(EnemyModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);
void EnemyModel_collisionInternal(compat_type(EnemyModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void EnemyModel_setAIState(compat_type(EnemyModel *) selfPtr, enum EnemyAIState state);
void EnemyModel_attack(compat_type(EnemyModel *) selfPtr);
void EnemyModel_takeDamage(compat_type(EnemyModel *) selfPtr, int amount, Direction fromDirection);
void EnemyModel_moveTowardPoint(compat_type(EnemyModel *) selfPtr, Vector2 point, float slop);

#endif
