/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/GamepadAxisVirtualButton.h"
#include "shell/Shell.h"
#include <stdlib.h>

GamepadAxisVirtualButton * GamepadAxisVirtualButton_create(struct Gamepad_device * device, unsigned int axisID, Atom actionID, int direction, float triggerThreshold, float releaseThreshold) {
	stemobject_create_implementation(GamepadAxisVirtualButton, init, device, axisID, actionID, direction, triggerThreshold, releaseThreshold)
}

void GamepadAxisVirtualButton_init(compat_type(GamepadAxisVirtualButton *) selfPtr, struct Gamepad_device * device, unsigned int axisID, Atom actionID, int direction, float triggerThreshold, float releaseThreshold) {
	GamepadAxisVirtualButton * self = selfPtr;
	
	StemObject_init(self);
	self->dispose = GamepadAxisVirtualButton_dispose;
	self->update = GamepadAxisVirtualButton_update;
	
	self->device = device;
	self->axisID = axisID;
	self->actionID = actionID;
	self->direction = direction;
	self->triggerThreshold = triggerThreshold;
	self->releaseThreshold = releaseThreshold;
	self->eventDispatcher = EventDispatcher_create(self);
	self->triggered = false;
}

void GamepadAxisVirtualButton_dispose(compat_type(GamepadAxisVirtualButton *) selfPtr) {
	GamepadAxisVirtualButton * self = selfPtr;
	
	self->eventDispatcher->dispose(self->eventDispatcher);
	StemObject_dispose(self);
}

void GamepadAxisVirtualButton_update(compat_type(GamepadAxisVirtualButton *) selfPtr, float value) {
	GamepadAxisVirtualButton * self = selfPtr;
	
	if (self->triggered) {
		if (value * self->direction < self->releaseThreshold * self->direction) {
			struct Gamepad_buttonEvent event;
			
			event.device = self->device;
			event.timestamp = Shell_getCurrentTime();
			event.buttonID = self->axisID;
			event.down = false;
			self->triggered = false;
			self->eventDispatcher->dispatchEvent(self->eventDispatcher, Atom_fromString(GAMEPAD_AXIS_VIRTUAL_BUTTON_EVENT_UP), &event);
		}
		
	} else if (value * self->direction > self->triggerThreshold * self->direction) {
		struct Gamepad_buttonEvent event;
		
		event.device = self->device;
		event.timestamp = Shell_getCurrentTime();
		event.buttonID = self->axisID;
		event.down = true;
		self->triggered = true;
		self->eventDispatcher->dispatchEvent(self->eventDispatcher, Atom_fromString(GAMEPAD_AXIS_VIRTUAL_BUTTON_EVENT_DOWN), &event);
	}
}
