/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAMEPAD_DETAIL_VIEW_H__
#define __GAMEPAD_DETAIL_VIEW_H__

typedef struct GamepadDetailView GamepadDetailView;

#include "3dmath/Matrix.h"
#include "gamepad/Gamepad.h"
#include "glgraphics/GLIncludes.h"
#include "stemobject/StemObject.h"

struct GamepadDetailView_mappableElement {
	bool axis;
	union {
		struct {
			unsigned int buttonID;
		} button;
		struct {
			unsigned int axisID;
			int direction;
		} axis;
	} u;
};

#define GamepadDetailView_structContents \
	StemObject_structContents \
	\
	struct Gamepad_device * device; \
	GLuint vboID; \
	GLuint eboID; \
	float width; \
	\
	struct GamepadDetailView_mappableElement (* mappableElementAtPosition)(compat_type(GamepadDetailView *) self, float position); \
	void (* draw)(compat_type(GamepadDetailView *) self, Matrix modelviewMatrix);

struct GamepadDetailView {
	GamepadDetailView_structContents
};

GamepadDetailView * GamepadDetailView_create(struct Gamepad_device * device);
void GamepadDetailView_init(compat_type(GamepadDetailView *) selfPtr, struct Gamepad_device * device);
void GamepadDetailView_dispose(compat_type(GamepadDetailView *) selfPtr);

struct GamepadDetailView_mappableElement GamepadDetailView_mappableElementAtPosition(compat_type(GamepadDetailView *) selfPtr, float position);
void GamepadDetailView_draw(compat_type(GamepadDetailView *) selfPtr, Matrix modelviewMatrix);

#endif
