/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __INVENTORY_MODEL_H__
#define __INVENTORY_MODEL_H__

typedef struct InventoryModel InventoryModel;

#include "stemobject/StemObject.h"
#include <stdlib.h>

typedef enum InventoryItemType {
	INVENTORY_ITEM_TYPE_HEAL_POWERUP,
	INVENTORY_ITEM_TYPE_KEY
} InventoryItemType;

struct InventoryModel_item {
	InventoryItemType type;
	union {
		struct {
			int rank;
		} healPowerup;
		struct {
			int keyID;
		} key;
	} u;
};

#define InventoryModel_structContents \
	StemObject_structContents \
	\
	size_t itemCount; \
	struct InventoryModel_item * items; \
	\
	compat_type(InventoryModel *) (* copy)(compat_type(InventoryModel *) self); \
	void (* addItem)(compat_type(InventoryModel *) self, InventoryItemType type, ...); \
	bool (* hasKey)(compat_type(InventoryModel *) self, int keyID); \
	void (* useKey)(compat_type(InventoryModel *) self, int keyID);

struct InventoryModel {
	InventoryModel_structContents
};

InventoryModel * InventoryModel_create();
InventoryModel * InventoryModel_createWithItems(size_t itemCount, struct InventoryModel_item * items);
void InventoryModel_init(compat_type(InventoryModel *) selfPtr);
void InventoryModel_initWithItems(compat_type(InventoryModel *) selfPtr, size_t itemCount, struct InventoryModel_item * items);
void InventoryModel_initCopy(compat_type(InventoryModel *) selfPtr, compat_type(InventoryModel *) originalPtr);
void InventoryModel_dispose(compat_type(InventoryModel *) selfPtr);

compat_type(InventoryModel *) InventoryModel_copy(compat_type(InventoryModel *) selfPtr);
void InventoryModel_addItem(compat_type(InventoryModel *) selfPtr, InventoryItemType type, ...);
bool InventoryModel_hasKey(compat_type(InventoryModel *) selfPtr, int keyID);
void InventoryModel_useKey(compat_type(InventoryModel *) selfPtr, int keyID);

#endif
