/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __MOVING_ENTITY_MODEL_H__
#define __MOVING_ENTITY_MODEL_H__

typedef struct MovingEntityModel MovingEntityModel;

#include "3dmath/Vector.h"
#include "convergence/GameObjectModel.h"
#include "convergence/SharedDefinitions.h"

#define MovingEntityModel_structContents \
	GameObjectModel_structContents \
	\
	Vector2 lastPosition; \
	Vector2 position; \
	Vector2 velocity; \
	Direction facing; \
	int moveIntentX; \
	int moveIntentY; \
	int facingIntentX; \
	int facingIntentY; \
	unsigned int facingIntentXDecay; \
	unsigned int facingIntentYDecay; \
	float radius; \
	float acceleration; \
	float deceleration; \
	float velocityMax; \
	\
	void (* protected_ivar(collisionInternal))(compat_type(MovingEntityModel *) self, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time); \
	void (* startMovingLeft)(compat_type(MovingEntityModel *) self); \
	void (* startMovingRight)(compat_type(MovingEntityModel *) self); \
	void (* startMovingDown)(compat_type(MovingEntityModel *) self); \
	void (* startMovingUp)(compat_type(MovingEntityModel *) self); \
	void (* stopMovingLeft)(compat_type(MovingEntityModel *) self); \
	void (* stopMovingRight)(compat_type(MovingEntityModel *) self); \
	void (* stopMovingDown)(compat_type(MovingEntityModel *) self); \
	void (* stopMovingUp)(compat_type(MovingEntityModel *) self); \
	void (* stopMoving)(compat_type(MovingEntityModel *) self);

struct MovingEntityModel {
	MovingEntityModel_structContents
};

MovingEntityModel * MovingEntityModel_create(DungeonRoomStateModel * stateModel);
void MovingEntityModel_init(compat_type(MovingEntityModel *) selfPtr, DungeonRoomStateModel * stateModel);
void MovingEntityModel_initCopy(compat_type(MovingEntityModel *) selfPtr, compat_type(MovingEntityModel *) originalPtr);
void MovingEntityModel_dispose(compat_type(MovingEntityModel *) selfPtr);

compat_type(MovingEntityModel *) MovingEntityModel_copy(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_update(compat_type(MovingEntityModel *) selfPtr);
bool MovingEntityModel_atRest(compat_type(MovingEntityModel *) selfPtr);
struct rect4f MovingEntityModel_roughBounds(compat_type(MovingEntityModel *) selfPtr);
struct collisionGeometry MovingEntityModel_collisionGeometry(compat_type(MovingEntityModel *) selfPtr);
bool MovingEntityModel_wouldCollide(compat_type(MovingEntityModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);
void MovingEntityModel_collision(compat_type(MovingEntityModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void MovingEntityModel_interpolate(compat_type(MovingEntityModel *) selfPtr, float time);
void MovingEntityModel_collisionInternal(compat_type(MovingEntityModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void MovingEntityModel_startMovingLeft(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_startMovingRight(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_startMovingDown(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_startMovingUp(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_stopMovingLeft(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_stopMovingRight(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_stopMovingDown(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_stopMovingUp(compat_type(MovingEntityModel *) selfPtr);
void MovingEntityModel_stopMoving(compat_type(MovingEntityModel *) selfPtr);

#endif
