/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __PLAYER_MODEL_H__
#define __PLAYER_MODEL_H__

typedef struct PlayerModel PlayerModel;

#include "convergence/GameSession.h"
#include "convergence/InventoryModel.h"
#include "convergence/MovingEntityModel.h"

#define PLAYER_EVENT_ATTACK_BIG "PLAYER_EVENT_ATTACK_BIG"
#define PLAYER_EVENT_ATTACK_SMALL "PLAYER_EVENT_ATTACK_SMALL"
#define PLAYER_EVENT_SHOOT "PLAYER_EVENT_SHOOT"
#define PLAYER_EVENT_SPY_SNEAK "PLAYER_EVENT_SPY_SNEAK"
#define PLAYER_EVENT_RANGER_LOCK "PLAYER_EVENT_RANGER_LOCK"
#define PLAYER_EVENT_DASH "PLAYER_EVENT_DASH"
#define PLAYER_EVENT_HIT "PLAYER_EVENT_HIT" // eventData -> int * damage
#define PLAYER_EVENT_DEATH "PLAYER_EVENT_DEATH"
#define PLAYER_EVENT_HEALED "PLAYER_EVENT_HEALED"
#define PLAYER_EVENT_GOT_KEY "PLAYER_EVENT_GOT_KEY"
#define PLAYER_EVENT_RANK_UP "PLAYER_EVENT_RANK_UP"
#define PLAYER_EVENT_EXITED "PLAYER_EVENT_EXITED" // eventData -> Atom exitName

#define PlayerModel_structContents \
	MovingEntityModel_structContents \
	\
	enum PlayerClass playerClass; \
	InventoryModel * inventory; \
	int rank; \
	int maxHealth; \
	int health; \
	int hitCooldown; \
	bool exited; \
	\
	void (* startAttack)(compat_type(PlayerModel *) selfPtr); \
	void (* startSpecialAction)(compat_type(PlayerModel *) selfPtr); \
	void (* stopAttack)(compat_type(PlayerModel *) selfPtr); \
	void (* stopSpecialAction)(compat_type(PlayerModel *) selfPtr); \
	void (* spotted)(compat_type(PlayerModel *) selfPtr, compat_type(EnemyModel *) enemy); \
	void (* protected_ivar(knockedBack))(compat_type(PlayerModel *) selfPtr, Vector2 knockback); \
	void (* protected_ivar(takeDamage))(compat_type(PlayerModel *) selfPtr, int amount, Direction fromDirection);

struct PlayerModel {
	PlayerModel_structContents
};

PlayerModel * PlayerModel_create(DungeonRoomStateModel * stateModel, Vector2 position, Direction facing);
void PlayerModel_init(compat_type(PlayerModel *) selfPtr, DungeonRoomStateModel * stateModel, Vector2 position, Direction facing);
void PlayerModel_initCopy(compat_type(PlayerModel *) selfPtr, compat_type(PlayerModel *) originalPtr);
void PlayerModel_dispose(compat_type(PlayerModel *) selfPtr);

compat_type(PlayerModel *) PlayerModel_copy(compat_type(PlayerModel *) selfPtr);
void PlayerModel_update(compat_type(PlayerModel *) selfPtr);
bool PlayerModel_atRest(compat_type(PlayerModel *) selfPtr);
bool PlayerModel_wouldCollide(compat_type(PlayerModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);
void PlayerModel_collisionInternal(compat_type(PlayerModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void PlayerModel_startAttack(compat_type(PlayerModel *) selfPtr);
void PlayerModel_stopAttack(compat_type(PlayerModel *) selfPtr);
void PlayerModel_startSpecialAction(compat_type(PlayerModel *) selfPtr);
void PlayerModel_stopSpecialAction(compat_type(PlayerModel *) selfPtr);
void PlayerModel_spotted(compat_type(PlayerModel *) selfPtr, compat_type(EnemyModel *) enemy);
void PlayerModel_knockedBack(compat_type(PlayerModel *) selfPtr, Vector2 knockback);
void PlayerModel_takeDamage(compat_type(PlayerModel *) selfPtr, int amount, Direction fromDirection);

#endif
