/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __PROJECTILE_MODEL_H__
#define __PROJECTILE_MODEL_H__

typedef struct ProjectileModel ProjectileModel;

#include "convergence/GameObjectModel.h"

#define PROJECTILE_EVENT_SHOT_STOPPED_WALL "PROJECTILE_EVENT_SHOT_STOPPED_WALL"

#define ProjectileModel_structContents \
	GameObjectModel_structContents \
	\
	Vector2 lastPosition; \
	Vector2 position; \
	Vector2 direction; \
	float radius; \
	float speed; \
	float travelDistance; \
	float distanceTraveled; \
	bool hostile; \
	int damage; \
	float knockback;

struct ProjectileModel {
	ProjectileModel_structContents
};

ProjectileModel * ProjectileModel_create(DungeonRoomStateModel * stateModel, Vector2 position, Vector2 direction, float speed, float travelDistance, bool hostile, int damage, float knockback);
void ProjectileModel_init(compat_type(ProjectileModel *) selfPtr, DungeonRoomStateModel * stateModel, Vector2 position, Vector2 direction, float speed, float travelDistance, bool hostile, int damage, float knockback);
void ProjectileModel_initCopy(compat_type(ProjectileModel *) selfPtr, compat_type(ProjectileModel *) originalPtr);
void ProjectileModel_dispose(compat_type(ProjectileModel *) selfPtr);

compat_type(ProjectileModel *) ProjectileModel_copy(compat_type(ProjectileModel *) selfPtr);
void ProjectileModel_update(compat_type(ProjectileModel *) selfPtr);
bool ProjectileModel_atRest(compat_type(ProjectileModel *) selfPtr);
struct rect4f ProjectileModel_roughBounds(compat_type(ProjectileModel *) selfPtr);
struct collisionGeometry ProjectileModel_collisionGeometry(compat_type(ProjectileModel *) selfPtr);
bool ProjectileModel_wouldCollide(compat_type(ProjectileModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);
void ProjectileModel_collision(compat_type(ProjectileModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void ProjectileModel_interpolate(compat_type(ProjectileModel *) selfPtr, float time);

#endif
