/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/ProjectileView.h"
#include "convergence/AudioManager.h"
#include "convergence/ProjectileModel.h"
#include <math.h>
#include <stddef.h>
#include <stdlib.h>

struct vertex_p2f {
	GLfloat position[2];
};

ProjectileView * ProjectileView_create(compat_type(ProjectileModel *) modelPtr) {
	stemobject_create_implementation(ProjectileView, init, modelPtr)
}

static bool shotStoppedWall(void * sender, Atom eventID, void * eventData, void * context) {
	AudioManager_play("plunk");
	return true;
}

void ProjectileView_init(compat_type(ProjectileView *) selfPtr, compat_type(ProjectileModel *) modelPtr) {
	ProjectileView * self = selfPtr;
	struct vertex_p2f vertices[3];
	GLushort indexes[3];
	ProjectileModel * model = modelPtr;
	
	GameObjectView_init(self, modelPtr);
	self->dispose = ProjectileView_dispose;
	self->draw = ProjectileView_draw;
	
	glGenBuffers(1, &self->vboID);
	glGenBuffers(1, &self->eboID);
	vertices[0].position[0] = 0.0f;
	vertices[0].position[1] = model->radius;
	vertices[1].position[0] = -sin(M_PI * 2 / 3) * model->radius;
	vertices[1].position[1] = cos(M_PI * 2 / 3) * model->radius;
	vertices[2].position[0] = -sin(M_PI * 4 / 3) * model->radius;
	vertices[2].position[1] = cos(M_PI * 4 / 3) * model->radius;
	indexes[0] = 0;
	indexes[1] = 1;
	indexes[2] = 2;
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(struct vertex_p2f) * 3, vertices, GL_STATIC_DRAW);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLushort) * 3, indexes, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
	self->model->eventDispatcher->registerForEvent(self->model->eventDispatcher, Atom_fromString(PROJECTILE_EVENT_SHOT_STOPPED_WALL), shotStoppedWall, self);
}

void ProjectileView_dispose(compat_type(ProjectileView *) selfPtr) {
	ProjectileView * self = selfPtr;
	
	glDeleteBuffers(1, &self->vboID);
	glDeleteBuffers(1, &self->eboID);
	self->model->eventDispatcher->unregisterForEvent(self->model->eventDispatcher, Atom_fromString(PROJECTILE_EVENT_SHOT_STOPPED_WALL), shotStoppedWall, self);
	GameObjectView_dispose(self);
}

void ProjectileView_draw(compat_type(ProjectileView *) selfPtr, Matrix modelviewMatrix, double deltaTime) {
	ProjectileView * self = selfPtr;
	ProjectileModel * model = (ProjectileModel *) self->model;
	
	Matrix_translate(&modelviewMatrix, model->position.x, model->position.y, 0.0f);
	Matrix_rotate(&modelviewMatrix, Vector3_init(0.0f, 0.0f, -1.0f), atan2(model->direction.x, model->direction.y));
	
	glBindBuffer(GL_ARRAY_BUFFER, self->vboID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self->eboID);
	
	glVertexPointer(2, GL_FLOAT, sizeof(struct vertex_p2f), (void *) offsetof(struct vertex_p2f, position));
	glEnableClientState(GL_VERTEX_ARRAY);
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelviewMatrix.m);
	glColor4ub(0xFF, 0xFF, 0xFF, 0xFF);
	
	glDrawElements(GL_TRIANGLES, 3, GL_UNSIGNED_SHORT, 0);
	
	glDisableClientState(GL_VERTEX_ARRAY);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}
