/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/RankUpPowerupModel.h"

RankUpPowerupModel * RankUpPowerupModel_create(DungeonRoomStateModel * stateModel, Vector2 position) {
	stemobject_create_implementation(RankUpPowerupModel, init, stateModel, position)
}

static void sharedInit(RankUpPowerupModel * self) {
	self->type = GAME_OBJECT_TYPE_RANK_UP_POWERUP;
	self->dispose = RankUpPowerupModel_dispose;
	self->copy = RankUpPowerupModel_copy;
	self->roughBounds = RankUpPowerupModel_roughBounds;
	self->collisionGeometry = RankUpPowerupModel_collisionGeometry;
	self->wouldCollide = RankUpPowerupModel_wouldCollide;
	self->collision = RankUpPowerupModel_collision;
}

void RankUpPowerupModel_init(compat_type(RankUpPowerupModel *) selfPtr, DungeonRoomStateModel * stateModel, Vector2 position) {
	RankUpPowerupModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->position = position;
}

void RankUpPowerupModel_initCopy(compat_type(RankUpPowerupModel *) selfPtr, compat_type(RankUpPowerupModel *) originalPtr) {
	RankUpPowerupModel * self = selfPtr;
	RankUpPowerupModel * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->position = original->position;
}

void RankUpPowerupModel_dispose(compat_type(RankUpPowerupModel *) selfPtr) {
	RankUpPowerupModel * self = selfPtr;
	
	StemObject_dispose(self);
}

compat_type(RankUpPowerupModel *) RankUpPowerupModel_copy(compat_type(RankUpPowerupModel *) selfPtr) {
	stemobject_copy_implementation(RankUpPowerupModel, initCopy)
}

struct rect4f RankUpPowerupModel_roughBounds(compat_type(RankUpPowerupModel *) selfPtr) {
	RankUpPowerupModel * self = selfPtr;
	struct rect4f bounds;
	
	bounds.left = self->position.x - RANK_INCREASE_POWERUP_RADIUS * RANK_INCREASE_POWERUP_RATIO;
	bounds.right = self->position.x + RANK_INCREASE_POWERUP_RADIUS * RANK_INCREASE_POWERUP_RATIO;
	bounds.bottom = self->position.y - RANK_INCREASE_POWERUP_RADIUS;
	bounds.top = self->position.y + RANK_INCREASE_POWERUP_RADIUS;
	return bounds;
}

struct collisionGeometry RankUpPowerupModel_collisionGeometry(compat_type(RankUpPowerupModel *) selfPtr) {
	RankUpPowerupModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->position.x - RANK_INCREASE_POWERUP_RADIUS * RANK_INCREASE_POWERUP_RATIO;
	geometry.u.sweptAABB.current.y = self->position.y - RANK_INCREASE_POWERUP_RADIUS;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = RANK_INCREASE_POWERUP_RADIUS * 2.0f * RANK_INCREASE_POWERUP_RATIO;
	geometry.u.sweptAABB.height = RANK_INCREASE_POWERUP_RADIUS * 2.0f;
	return geometry;
}

bool RankUpPowerupModel_wouldCollide(compat_type(RankUpPowerupModel *) selfPtr, compat_type(GameObjectModel *) objectPtr) {
	RankUpPowerupModel * self = selfPtr;
	GameObjectModel * object = objectPtr;
	
	switch (object->type) {
		case GAME_OBJECT_TYPE_PLAYER:
			return !self->markedForRemoval;
		default:
			return false;
	}
}

void RankUpPowerupModel_collision(compat_type(RankUpPowerupModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time) {
	RankUpPowerupModel * self = selfPtr;
	
	self->markedForRemoval = true;
}
