/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/TextureCanvas.h"
#include "serialization/DeserializationContext.h"
#include "serialization/SerializationContext.h"
#include <string.h>

TextureCanvas * TextureCanvas_create() {
	stemobject_create_implementation(TextureCanvas, init)
}

void TextureCanvas_init(compat_type(TextureCanvas *) selfPtr) {
	TextureCanvas * self = selfPtr;
	
	StemObject_init(self);
	self->dispose = TextureCanvas_dispose;
}

void TextureCanvas_dispose(compat_type(TextureCanvas *) selfPtr) {
	TextureCanvas * self = selfPtr;
	
	free(self->textureFileName);
	free(self->tiles);
	StemObject_dispose(self);
}

TextureCanvas * TextureCanvas_deserialize(compat_type(DeserializationContext *) deserializationContext) {
	stemobject_deserialize_implementation(TextureCanvas, deserializationContext, loadSerializedData)
}

bool TextureCanvas_loadSerializedData(compat_type(TextureCanvas *) selfPtr, compat_type(DeserializationContext *) deserializationContext) {
	TextureCanvas * self = selfPtr;
	DeserializationContext * context = deserializationContext;
	uint16_t formatVersion;
	const char * textureFileName;
	size_t tileCount, tileIndex;
	struct TextureCanvas_tile * tiles;
	const char * tileName;
	
	context->beginStructure(context, "texture_canvas");
	formatVersion = context->readUInt16(context, "format_version");
	if (context->status != SERIALIZATION_ERROR_OK || formatVersion != TEXTURE_CANVAS_SERIALIZATION_FORMAT_VERSION) {
		return false;
	}
	textureFileName = context->readString(context, "texture");
	tileCount = context->beginDictionary(context, "tiles");
	tiles = malloc(sizeof(struct TextureCanvas_tile) * tileCount);
	for (tileIndex = 0; tileIndex < tileCount; tileIndex++) {
		tileName = context->readNextDictionaryKey(context);
		tiles[tileIndex].name = Atom_fromString(tileName);
		context->beginStructure(context, tileName);
		context->beginStructure(context, "bounds");
		tiles[tileIndex].bounds.left = context->readFloat(context, "left");
		tiles[tileIndex].bounds.right = context->readFloat(context, "right");
		tiles[tileIndex].bounds.bottom = context->readFloat(context, "bottom");
		tiles[tileIndex].bounds.top = context->readFloat(context, "top");
		context->endStructure(context);
		context->endStructure(context);
	}
	context->endDictionary(context);
	context->endStructure(context);
	
	if (context->status != SERIALIZATION_ERROR_OK) {
		free(tiles);
		return false;
	}
	
	TextureCanvas_init(self);
	self->textureFileName = malloc(strlen(textureFileName) + 1);
	strcpy(self->textureFileName, textureFileName);
	self->tileCount = tileCount;
	self->tiles = tiles;
	
	return true;
}

void TextureCanvas_serialize(compat_type(TextureCanvas *) selfPtr, compat_type(SerializationContext *) serializationContext) {
	TextureCanvas * self = selfPtr;
	SerializationContext * context = serializationContext;
	size_t tileIndex;
	
	context->beginStructure(context, "texture_canvas");
	context->writeUInt16(context, "format_version", TEXTURE_CANVAS_SERIALIZATION_FORMAT_VERSION);
	context->writeString(context, "texture", self->textureFileName);
	context->beginDictionary(context, "tiles");
	for (tileIndex = 0; tileIndex < self->tileCount; tileIndex++) {
		context->beginStructure(context, self->tiles[tileIndex].name);
		context->beginStructure(context, "bounds");
		context->writeFloat(context, "left", self->tiles[tileIndex].bounds.left);
		context->writeFloat(context, "right", self->tiles[tileIndex].bounds.right);
		context->writeFloat(context, "bottom", self->tiles[tileIndex].bounds.bottom);
		context->writeFloat(context, "top", self->tiles[tileIndex].bounds.top);
		context->endStructure(context);
		context->endStructure(context);
	}
	context->endDictionary(context);
	context->endStructure(context);
}
